@@ -264,17 +264,20 @@ class DNSService(object):
                         break
                     payload += data
 
+            # NOTE: Any uncaught exceptions will result in the main loop
+            # ending unexpectedly. Ensure proper ordering of blocks, and
+            # ensure no exceptions are generated from within.
+            except socket.timeout:
+                client.close()
+                LOG.warn(_LW("TCP Timeout from: %(host)s:%(port)d") %
+                         {'host': addr[0], 'port': addr[1]})
+
             except socket.error as e:
                 client.close()
                 errname = errno.errorcode[e.args[0]]
                 LOG.warn(_LW("Socket error %(err)s from: %(host)s:%(port)d") %
                          {'host': addr[0], 'port': addr[1], 'err': errname})
 
-            except socket.timeout:
-                client.close()
-                LOG.warn(_LW("TCP Timeout from: %(host)s:%(port)d") %
-                         {'host': addr[0], 'port': addr[1]})
-
             except struct.error:
                 client.close()
                 LOG.warn(_LW("Invalid packet from: %(host)s:%(port)d") %
@@ -335,7 +338,7 @@ class DNSService(object):
                         # Handle TCP Responses
                         msg_length = len(response)
                         tcp_response = struct.pack("!H", msg_length) + response
-                        client.send(tcp_response)
+                        client.sendall(tcp_response)
                     else:
                         # Handle UDP Responses
                         self._dns_sock_udp.sendto(response, addr)
