@@ -13,6 +13,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import logging
 import uuid
 
 import oslo_messaging as messaging
@@ -24,9 +25,13 @@ from mock import patch
 from designate import exceptions
 from designate import objects
 from designate.backend import impl_fake
+from designate.storage.impl_sqlalchemy import tables
 from designate.central import rpcapi as central_rpcapi
 from designate.mdns import rpcapi as mdns_rpcapi
 from designate.tests.test_pool_manager import PoolManagerTestCase
+import designate.pool_manager.service as pm_module
+
+LOG = logging.getLogger(__name__)
 
 
 class PoolManagerServiceNoopTest(PoolManagerTestCase):
@@ -119,7 +124,7 @@ class PoolManagerServiceNoopTest(PoolManagerTestCase):
     def _build_domains(self, n, action, status):
         return [
             self._build_domain("zone%02X.example" % cnt, action,
-                             status, id=str(uuid.uuid4()))
+                               status, id=str(uuid.uuid4()))
             for cnt in range(n)
         ]
 
@@ -475,7 +480,7 @@ class PoolManagerServiceNoopTest(PoolManagerTestCase):
                            mock_cent_update_status, *a):
         self.service.update_domain = Mock()
         mock_find_domains.return_value = self._build_domains(2, 'UPDATE',
-                                                         'PENDING')
+                                                             'PENDING')
         self.service.periodic_sync()
 
         self.assertEqual(1, mock_find_domains.call_count)
@@ -484,36 +489,166 @@ class PoolManagerServiceNoopTest(PoolManagerTestCase):
         self.assertEqual(2, self.service.update_domain.call_count)
         self.assertEqual(0, mock_cent_update_status.call_count)
 
+    @patch.object(pm_module.time, 'sleep')
     @patch.object(mdns_rpcapi.MdnsAPI, 'notify_zone_changed')
     @patch.object(central_rpcapi.CentralAPI, 'update_status')
     @patch.object(central_rpcapi.CentralAPI, 'find_domains')
-    def test_periodic_sync_with_failing_update(self, mock_find_domains,
-                                               mock_cent_update_status, *a):
+    def test_periodic_sync_with_failing_update(
+            self, mock_find_domains, mock_cent_update_status, *mocks):
         self.service.update_domain = Mock(return_value=False)  # fail update
         mock_find_domains.return_value = self._build_domains(3, 'UPDATE',
-                                                         'PENDING')
+                                                             'PENDING')
         self.service.periodic_sync()
 
         self.assertEqual(1, mock_find_domains.call_count)
         criterion = mock_find_domains.call_args_list[0][0][1]
         self.assertEqual('!ERROR', criterion['status'])
-        # all zones are now in ERROR status
-        self.assertEqual(3, self.service.update_domain.call_count)
+
+        # 3 zones, all failing, with 3 attempts: 9 calls
+        self.assertEqual(9, self.service.update_domain.call_count)
+
+        # the zones have been put in ERROR status
         self.assertEqual(3, mock_cent_update_status.call_count)
 
+    @patch.object(pm_module.time, 'sleep')
     @patch.object(mdns_rpcapi.MdnsAPI, 'notify_zone_changed')
     @patch.object(central_rpcapi.CentralAPI, 'update_status')
     @patch.object(central_rpcapi.CentralAPI, 'find_domains')
     def test_periodic_sync_with_failing_update_with_exception(
-            self, mock_find_domains, mock_cent_update_status, *a):
+            self, mock_find_domains, mock_cent_update_status, *mocks):
         self.service.update_domain = Mock(side_effect=Exception)
         mock_find_domains.return_value = self._build_domains(3, 'UPDATE',
-                                                         'PENDING')
+                                                             'PENDING')
         self.service.periodic_sync()
 
         self.assertEqual(1, mock_find_domains.call_count)
         criterion = mock_find_domains.call_args_list[0][0][1]
         self.assertEqual('!ERROR', criterion['status'])
-        # the first updated zone is now in ERROR status
-        self.assertEqual(1, self.service.update_domain.call_count)
-        self.assertEqual(1, mock_cent_update_status.call_count)
+
+        # 3 zones, all failing, with 3 attempts: 9 calls
+        self.assertEqual(9, self.service.update_domain.call_count)
+
+        # the zones have been put in ERROR status
+        self.assertEqual(3, mock_cent_update_status.call_count)
+
+    # Periodic recovery
+
+    @patch.object(pm_module.time, 'sleep')
+    @patch.object(mdns_rpcapi.MdnsAPI, 'notify_zone_changed')
+    @patch.object(central_rpcapi.CentralAPI, 'update_status')
+    def test_periodic_recovery(self, mock_find_domains,
+                               mock_cent_update_status, *mocks):
+
+        def mock_get_failed_domains(ctx, action):
+            if action == pm_module.DELETE_ACTION:
+                return self._build_domains(3, 'DELETE', 'ERROR')
+            if action == pm_module.CREATE_ACTION:
+                return self._build_domains(4, 'CREATE', 'ERROR')
+            if action == pm_module.UPDATE_ACTION:
+                return self._build_domains(5, 'UPDATE', 'ERROR')
+
+        self.service._get_failed_domains = mock_get_failed_domains
+        self.service.delete_domain = Mock()
+        self.service.create_domain = Mock()
+        self.service.update_domain = Mock()
+
+        self.service.periodic_recovery()
+
+        self.assertEqual(3, self.service.delete_domain.call_count)
+        self.assertEqual(4, self.service.create_domain.call_count)
+        self.assertEqual(5, self.service.update_domain.call_count)
+
+
+class PoolManagerServiceEndToEndTest(PoolManagerServiceNoopTest):
+
+    def setUp(self):
+        super(PoolManagerServiceEndToEndTest, self).setUp()
+
+    def _fetch_all_domains(self):
+        """Fetch all zones including deleted ones
+        """
+        query = tables.zones.select()
+        return self.storage.session.execute(query).fetchall()
+
+    def _log_all_domains(self, zones, msg=None):
+        """Log out a summary of zones
+        """
+        if msg:
+            LOG.debug("--- %s ---" % msg)
+        cols = ('name', 'status', 'action', 'deleted', 'deleted_at',
+                'parent_domain_id')
+        tpl = "%-35s | %-11s | %-11s | %-32s | %-20s | %s"
+        LOG.debug(tpl % cols)
+        for z in zones:
+            LOG.debug(tpl % tuple(z[k] for k in cols))
+
+    def _assert_count_all_domains(self, n):
+        """Assert count ALL zones including deleted ones
+        """
+        zones = self._fetch_all_domains()
+        if len(zones) == n:
+            return
+
+        msg = "failed: %d zones expected, %d found" % (n, len(zones))
+        self._log_all_domains(zones, msg=msg)
+        raise Exception("Unexpected number of zones")
+
+    def _assert_num_failed_domains(self, action, n):
+        zones = self.service._get_failed_domains(
+            self.admin_context, action)
+        if len(zones) != n:
+            LOG.error("Expected %d failed zones, got %d", n, len(zones))
+            self._log_all_domains(zones, msg='listing zones')
+            self.assertEqual(n, len(zones))
+
+    def _assert_num_healthy_domains(self, action, n):
+        criterion = {
+            'action': action,
+            'pool_id': pm_module.CONF['service:pool_manager'].pool_id,
+            'status': '!%s' % pm_module.ERROR_STATUS
+        }
+        zones = self.service.central_api.find_domains(self.admin_context,
+                                                      criterion)
+        if len(zones) != n:
+            LOG.error("Expected %d healthy zones, got %d", n, len(zones))
+            self._log_all_domains(zones, msg='listing zones')
+            self.assertEqual(n, len(zones))
+
+    @patch.object(mdns_rpcapi.MdnsAPI, 'notify_zone_changed')
+    def test_periodic_sync_and_recovery(
+            self, mock_cent_update_status, *a):
+        # Periodic sync + recovery
+        self.service._periodic_sync_retry_interval = 0
+
+        # Create healthy zones, run a periodic sync that will fail
+        self.create_domain(name='created.example.com.')
+        self._assert_num_healthy_domains(pm_module.CREATE_ACTION, 1)
+
+        z = self.create_domain(name='updated.example.net.')
+        z.email = 'info@example.net'
+        self.service.central_api.update_domain(self.admin_context, z)
+        self._assert_num_healthy_domains(pm_module.UPDATE_ACTION, 1)
+
+        with patch.object(self.service, '_update_domain_on_target',
+                          return_value=False):
+            self.service.periodic_sync()
+
+        zones = self.service._fetch_healthy_zones(self.admin_context)
+        self.assertEqual(0, len(zones))
+        self._assert_num_failed_domains(pm_module.CREATE_ACTION, 1)
+        self._assert_num_failed_domains(pm_module.UPDATE_ACTION, 1)
+
+        # Now run a periodic_recovery that will fix the zones
+
+        backends = self.service.target_backends
+        for tid in self.service.target_backends:
+            backends[tid].create_domain = Mock()
+            backends[tid].update_domain = Mock()
+            backends[tid].delete_domain = Mock()
+
+        self.service.periodic_recovery()
+
+        # There are 2 pool targets in use
+        for backend in self.service.target_backends.itervalues():
+            self.assertEqual(1, backend.create_domain.call_count)
+            self.assertEqual(1, backend.update_domain.call_count)
