@@ -19,46 +19,13 @@ import uuid
 import mock
 from oslotest import base as test
 from oslo_utils import timeutils
-import six
 import testtools
 
 from designate.central import rpcapi as central_api
 from designate import context
 from designate import rpc
 from designate.zone_manager import tasks
-
-
-class RoObject(object):
-    """Read-only object: raise exception on unexpected
-    __setitem__ or __setattr__
-    """
-    def __init__(self, d=None, **kw):
-        if d:
-            kw.update(d)
-
-        self.__dict__.update(kw)
-
-    def __getitem__(self, k):
-        try:
-            return self.__dict__[k]
-        except KeyError:
-            raise NotImplementedError(
-                "Attempt to perform __getitem__"
-                " %r on RoObject %r" % (k, self.__dict__)
-            )
-
-    def __setitem__(self, k, v):
-        raise NotImplementedError(
-            "Attempt to perform __setitem__ or __setattr__"
-            " %r on RoObject %r" % (k, self.__dict__)
-        )
-
-    def __setattr__(self, k, v):
-        self.__setitem__(k, v)
-
-    def __iter__(self):
-        for k in six.iterkeys(self.__dict__):
-            yield k, self.__dict__[k]
+from designate.tests.unit import RoObject
 
 
 class DummyTask(tasks.PeriodicTask):
@@ -72,6 +39,7 @@ class TaskTest(test.BaseTestCase):
 
         self.opts = opts
         self.opts_patcher = mock.patch('oslo_config.cfg.CONF')
+        self.addCleanup(self.opts_patcher.stop)
 
         self.opts_mock = self.opts_patcher.start()
         self.opts_mock.__getitem__.side_effect = lambda n: opts[n]
@@ -156,12 +124,14 @@ class PeriodicExistsTest(TaskTest):
         self.ctxt = mock.Mock()
         get_admin_ctxt_patcher = mock.patch.object(context.DesignateContext,
                                             'get_admin_context')
+        self.addCleanup(get_admin_ctxt_patcher.stop)
         get_admin_context = get_admin_ctxt_patcher.start()
         get_admin_context.return_value = self.ctxt
 
         # Patch get_notifier so that it returns a mock..
         get_notifier_patcher = mock.patch.object(rpc, 'get_notifier')
         get_notifier = get_notifier_patcher.start()
+        self.addCleanup(get_notifier_patcher.stop)
         self.mock_notifier = mock.Mock()
         get_notifier.return_value = self.mock_notifier
 
@@ -172,11 +142,12 @@ class PeriodicExistsTest(TaskTest):
         # correct below
         self.period = tasks.PeriodicExistsTask._get_period(2)
         self.period_data = {
-            "audit_period_beginning": str(self.period[0]),
-            "audit_period_ending": str(self.period[1])
+            "audit_period_beginning": self.period[0],
+            "audit_period_ending": self.period[1]
         }
         get_period_patcher = mock.patch.object(
             tasks.PeriodicExistsTask, '_get_period')
+        self.addCleanup(get_period_patcher.stop)
         self.get_period = get_period_patcher.start()
         self.get_period.return_value = self.period
 
@@ -229,6 +200,7 @@ class PeriodicSecondaryRefreshTest(TaskTest):
         self.ctxt = mock.Mock()
         get_admin_ctxt_patcher = mock.patch.object(context.DesignateContext,
                                                    'get_admin_context')
+        self.addCleanup(get_admin_ctxt_patcher.stop)
         get_admin_context = get_admin_ctxt_patcher.start()
         get_admin_context.return_value = self.ctxt
 
@@ -236,6 +208,7 @@ class PeriodicSecondaryRefreshTest(TaskTest):
         self.central = mock.Mock()
         get_central_patcher = mock.patch.object(central_api.CentralAPI,
                                         'get_instance')
+        self.addCleanup(get_central_patcher.stop)
         get_central = get_central_patcher.start()
         get_central.return_value = self.central
 
